import {
  AfterContentInit,
  Directive,
  ElementRef,
  HostListener,
  Input,
  Renderer2,
} from "@angular/core";

@Directive({
  selector: "[resizable]",
})
export class ResizableDirective implements AfterContentInit {
  private readonly resizableElementClass = "resizable-element";
  private readonly resizerClass = "resizer";

  private resizer: any;
  private firstElement: any;
  private secondElement: any;
  private coveringPanel: any;
  private isHandlerDragging = false;

  @Input() firstElementMinWidth = 100;
  @Input() secondElementMinWidth = 100;
  @Input() firstElementMinWidthUnit: "px" | "%" = "px";
  @Input() secondElementMinWidthUnit: "px" | "%" = "px";

  constructor(
    private readonly ref: ElementRef,
    private readonly renderer: Renderer2
  ) {}

  ngAfterContentInit(): void {
    const elements = this.ref.nativeElement.getElementsByClassName(
      this.resizableElementClass
    );
    if (elements.length % 2 !== 0) {
      return;
    }

    this.createCoveringPanel();

    this.firstElement = elements[0];
    this.secondElement = elements[1];

    this.renderer.setStyle(
      this.firstElement,
      "min-width",
      this.firstElementMinWidth + this.firstElementMinWidthUnit
    );
    this.renderer.setStyle(
      this.secondElement,
      "min-width",
      this.secondElementMinWidth + this.secondElementMinWidthUnit
    );

    this.resizer = this.renderer.createElement("div");
    this.renderer.addClass(this.resizer, this.resizerClass);
    this.renderer.insertBefore(
      this.ref.nativeElement,
      this.resizer,
      this.firstElement.nextSibling
    );

    this.resizer.addEventListener(
      "mousedown",
      (e: { preventDefault: () => void; stopPropagation: () => void }) => {
        e.preventDefault();
        e.stopPropagation();

        this.addCoveringPanel();

        this.isHandlerDragging = true;
      }
    );
  }

  @HostListener("mouseup", ["$event"])
  onMouseUp(e: MouseEvent): void {
    e.preventDefault();
    e.stopPropagation();

    this.removeCoveringPanel();

    this.isHandlerDragging = false;
  }

  @HostListener("mousemove", ["$event"])
  onMouseMove(e: MouseEvent): void {
    if (!this.isHandlerDragging) {
      return;
    }

    e.preventDefault();
    e.stopPropagation();

    const pointerRelativeXPos = e.clientX - this.ref.nativeElement.offsetLeft;

    this.renderer.setStyle(
      this.firstElement,
      "width",
      pointerRelativeXPos - 8 + "px"
    );
    this.renderer.setStyle(
      this.secondElement,
      "width",
      this.ref.nativeElement.offsetWidth -
        this.firstElement.offsetWidth -
        this.resizer.offsetWidth +
        "px"
    );
    this.renderer.setStyle(this.firstElement, "flex-grow", 0);
  }

  /**
   * Créé une div au dessus des éléments pouvant être resize afin d'assurer que les events soient bien trigger
   * dans le cas où la souris passe au dessus d'embeds par exemple
   * @private
   */
  private createCoveringPanel(): void {
    this.coveringPanel = this.renderer.createElement("div");
    this.renderer.setStyle(this.coveringPanel, "position", "absolute");
    this.renderer.setStyle(this.coveringPanel, "z-index", 100);
    this.renderer.setStyle(this.coveringPanel, "width", "100%");
    this.renderer.setStyle(this.coveringPanel, "height", "100%");
    this.renderer.setStyle(this.coveringPanel, "cursor", "ew-resize");
  }

  private addCoveringPanel(): void {
    if (!this.ref.nativeElement.contains(this.coveringPanel)) {
      this.renderer.appendChild(this.ref.nativeElement, this.coveringPanel);
    }
  }

  private removeCoveringPanel(): void {
    if (this.ref.nativeElement.contains(this.coveringPanel)) {
      this.renderer.removeChild(this.ref.nativeElement, this.coveringPanel);
    }
  }
}
